/* Custom mainline for macro */

        signal _VREMain


_VREMain:
/*:VRX         Main
*/
/* Replace the Password entries for current Realm */
/* cgi V1.1 program */
    /* Realm access file has format:AuthName.acc
        AuthUserFile path/filename [ If * then in current directory realm.pwd else URL ]
        AuthGroupFile path/filename
        [AuthName realm]
        AuthType Basic
        
        <Limit GET [PUT POST] >
        Order allow/deny [deny/allow]
        Allow from all [ Net1. Net2. ]
        Deny from all [ Net1. Net2. ]
        </Limit>

      Password file has format:
        userid password
        ...

    */
/*  
    Input argument
        RealmName
        UserList
    Produces object 
        Domain
        RealmName
        UserAuthFile
        UserList

    Uses Template GetRealmPwd.w3t in current directory
*/

        

Main:
    signal on halt


    /* Read the environment variables */
    call get_env 


/* Read input data and build SIBOM data in Data */
    If Global.!Request_Method = 'POST' then do
        len =Global.!Content_Length
        Data = ''

        do while length(Data) < len         /* ++++ */
            Data = Data||linein()
        end        
        Data = GetBody(Data)
    end
    Else do
        parse arg Data /* Get the argument */
        Data = strip(Data)
        If Data == '' | Data == '""' then do
            /* get the Query_string value from the environment */
            Data=Global.!Query_String
        end
        
        Data = GetBody(Data)
    end

    handle = 0
    If \GetAttrVal('RealmName',Data,'Realm','type','handle') then do
        say 'No realm name specified'
        return 1
    end
    rc = GetAttrVal('UserList',Data,'UserList','type')


    /* Now read the Globals file for this server */
    If \ Read_Global() then do
        /* Globals file error */
        say 'Unable to read 'Global.!Rules
        return 1
    end

    rc = GetAttrVal('Domain',data,'domain','type')

    /* Find the short_name for this domain */
    parse var Global.!Globals 'Domains:' Global.!DomainList ':EndDomains'
    Global.!DomainList = Translate(Global.!DomainList,,'090a0d'x) 
    found = 0
    Do i = 1 to words(Global.!DomainList)
        /* Find the domain we want */
        entry = word(Global.!DomainList,i)
        parse var entry ip ',' short_name ',' domain_name
        If domain == short_name then do
            found = 1
            leave
        end
    end
    If \ found then do
        say 'Domain 'domain' not found'
        return 1
        /* Domain_name not found */
    end
    /* Verify that the user is in Realm "Admin" */
    If RealmAuth(Global.!Auth_Realm, Global.Remote_User, Global.!Server_Name Domain_name) then return 1

    /* Find all the realms for this domain */
    /* realms are in the rules directory in sub_directory "short_name" */
    dir = Global.!Rules||short_name
    /* read all the .acc file names */
    rc = SysFileTree(dir||'\*.acc', acc , 'F')
    error = 1
    Do i = 1 to acc.0
        /* date time size attr file */
        file = word(acc.i,5)
        position  = LastPos( "\", file )
        filen = substr(file,position+1)
        parse var filen filen '.' .
        If Filen = Realm then do
            i = acc.0
            error = 0
        end
    end
    If error then do
        say 'Realm account file for 'Realm 'not found'
        return 1
    end

    /* Read the Realm account file */
    length = Stream( file, "Command", "Query Size" )
    Account  = CharIn( file, 1, length )
    strm_status = Stream( file, "Command", "Close" )
        
    /* Parse the data to extract the variables */
    Parse var Account 'AuthUserFile' AuthUserFile '0d0a'x  'AuthGroupFile' AuthGroupFile '0d0a'x  '<Limit' Limits '>'  'Order' Order '0d0a'x  'Allow from' Allow '0d0a'x  'Deny from' Deny '0d0a'x
    AuthUserFile=Strip(AuthUserFile)
    AuthGroupFile=Strip(AuthGroupFile)
    Limits=Strip(Limits)
    Order=Strip(Order)
    Allow=Strip(Allow)
    Deny=Strip(Deny)

    /* If the realm requested is '*' then create a new realm */
    If AuthUserFile == '*' then do
        PwdFile = dir||'\'Realm'.pwd'
    end
    Else do
        interpret "parse var Global.!Globals 'Alias_"short_name':'"' Global.!Alias ':EndAlias_"short_name"'"
        Global.!Alias = Translate(Global.!Alias,,'090a0d'x) 
        PwdFile = Trans_URL(AuthUserFile)
    end

    /* Add some basic data */
    /*rc = AddAttrVal('RealmName','Data',Realm,t_string)*/ 
    rc = AddAttrVal('AuthUserFile','Data',AuthUserFile,t_string)

    /* Backup the Password file (If it exists ) */
    /* read all the file names */
    Position = LastPos('.',PwdFile)
    If position <> 0 then Scan = Substr(PwdFile,1,position-1)
    else Scan = PwdFile
    rc = SysFileTree(Scan||'*', acc , 'F')
    NewSuffx = '001'
    Do i = 1 to acc.0
        /* date time size attr file */
        file = word(acc.i,5)
        position  = LastPos( ".", file )
        suffx = substr(file,position+1)
        If Datatype( suffx, "N" ) & suffx >= NewSuffx then NewSuffx = Right(Suffx+1,3,'0')
    end
    'Copy 'PwdFile Scan||'.'||NewSuffx
    'Del 'PwdFile 

  
    /* Now write the new Passwordfile */
    UserList = Strip(UserList)
    UserList=Translate(UserList,' ','0d'x)
    Do while UserList <> ''
        Parse var UserList userid password '0a'x UserList
        userid = Strip(userid)
        password = Strip(password)
        If UserId <> '' & password <> '' then do          
            call lineout PwdFile, userid||' '||password||';'
        end
    end


    /* Now get the template to build the output */
    If Get_Template('GetRealmPwd') then do
        Parse var Txn.!Template . 'BODY:' template ':End'
        output=FillData(Template, Data)
    end
    /* Output the data */
    say 'CONTENT-TYPE: text/html'
    /* say 'CONTENT-LENGTH: 'length(output) */
    say
    say output
    call lineout /* Close output */

return 0
/*:VRX         AddAttrVal
*/
AddAttrVal: 
/* Add an object or item to a packed object */
parse arg !_attr, !_object, !_Value, !_Type
/* 
    Arguments
        attr -  Literal name of the attribute
        object- Quoted name of the variable containing the object
        value-  value the attribute is to be set to
        type -  Data type of the attribute (string or object)
    Returns
        1 OK
        0 Error
*/


    /* Add the new attribute */
    !_Item = !_attr||'='||!_value
    !_ItemBinaryLen = X2C(D2X(length(!_Item)+1,8))
    !_NewAttr = !_ItemBinaryLen||!_type||!_item
    rc=Value(!_object,Value(!_object)||!_NewAttr)
    !_FOUND = 1


drop !_attr !_object !_Value !_Type
drop !_ItemBinaryLen !_NewAttr !_Item 
return !_FOUND



/*:VRX         FillData
*/
FillData: procedure expose Global. TXN. Data
parse arg template, object
/*say 'FillData Start'  Time( "E" ) */

next = 1
output = ''
If template == '' then next = 0

Do While template <> ''
    Parse var template text '&' item '&' template
    output=output||text
    /* Test for commands */
    If item <> '' then do
        Parse var item command '(' arguments ')'
        Parse var arguments  exp ',' _Strings 
        If _Strings <> '' then _Strings = substr(_Strings,1,length(_Strings)-1)
        If Substr(_Strings,1,1) == '"' then do
            Parse var _Strings '"' NullString '"' . ',' NotNullString
        end
        Else do
            Parse var _Strings Nullstring ',' NotNullString 
        end
        If Substr(NotNullString,1,1) == '"' then do
            Parse var NotNullString '"' NotNullString '"'
        end
        handle = 0

        select
            when substr(command,1,1) == '#' then do
                arguments = Translate(arguments,'&','%')
                arguments = FillData(arguments, Data)

                REXX = Substr(Command,2)
                /* Check the script is present */
                RexxFile = REXX||'.cmd'
                RexxFile = Stream( RexxFile, "Command", "Query Exists" )
                If RexxFile == '' then do
                    /* Command file not found */
                    say RexxFile 'Does not exist'
                    output = output||'&'||item
                    template = '&'||template /* put the ampersands back! */
                end
                else do
                    Rexxit = "Call '"RexxFile"' arguments"
                    Interpret Rexxit 
                    /* Result has format rc ResultValue */

                    Parse var result rc value
                    If rc <> 1 then do
                        say Rexxfile 'rc='rc result
                        output = output||'&'||item
                        template = '&'||template /* put the ampersands back! */
                    end
                    /* Then set result as value of field */
                    Else do /* Re-Parse the result of the script */
                        template = value||template
                    end
                end
            end

            when Command == 'IFNULL' then do
                /* If command */
                rc = GetAttrVal(exp, object, 'value')               
                If value == '' then do
                    /* Convert % to &  to allow embedded fields */
                    NewString=Translate(NullString,'&','%')
                    template = NewString||template
                end
                else do
                    /* Convert % to &  to allow embedded fields */
                    NewString=Translate(NotNullString,'&','%')
                    template = NewString||template
                end
            end
            when command == 'LIST' then do
                NewString = ''
                Do While GetAttrVal(exp, object, 'value', 'value_type', 'handle')
                    NewString = NewString||NullString||value||NotNullString
                end
                template = NewString||template
            end
            when command = 'HTMLPARA' then do
                NewString = ''
                rc=GetAttrVal(exp, object, 'value', 'value_type', 'handle')
                do while value <> ''
                    parse var value line '0d0a'x value
                    NewString = NewString||NullString||Line||NotNullString
                end
                output = output||NewString
            end
            when command = 'URLSAFE' then do
                NewString = ''
                rc=GetAttrVal(exp, object, 'value', 'value_type', 'handle')
                Newstring = URL_SAFE(value)
                output = output||NewString
            end
            when command = "HTMLSAFE" then do
                NewString = ''
                rc=GetAttrVal(exp, object, 'value', 'value_type', 'handle')
                Newstring = HTML_SAFE(value)
                output = output||NewString
            end
            when command = "SELECT" then do
                /* Build SELECT and OPTION list arg1 has Value/Name; pairs */
                NewString = '<SELECT NAME="'exp'" > 'Global.!endline
                
                rc = GetAttrVal(exp,object,'Value','value_type','handle')
                value = Strip(value)
                Do while nullstring <> ''
                    parse var nullstring option ';' nullstring
                    parse var option option '/' text
                    option = Strip(Option)
                    text = Strip(text)
                    selected = ''
                    If value == option then selected = 'SELECTED'
                    If text == '' then text = option
                    NewString = Newstring||'<OPTION VALUE="'option'"' selected'>'||text||Global.!endline
                end
                Output = Output||Newstring||'</SELECT>'||Global.!endline
            end
            otherwise do
                NewString = ''
                Do While (length(item) < 32) & GetAttrVal(item, object, 'value', 'value_type', 'handle') 
                    /* we have the substitution variable */
                    if value_type = Global.!Type_Object then do
                        /* Look for template for item */
                        command = "parse var TXN.!Template junk '"'0d0a'x||Item||':'"' NewTemplate ':End' "
                        interpret command
                        NewString = NewString||FillData(NewTemplate, value)
                    end
                    else do /* just put the value into the output! */
                        NewString = NewString||value
                    end
                end /* processed all instances of this item name */
                if handle = 0 then do 
                    /* If we have not found at least 1 instance */
                    output = output||'&'||item
                    template = '&'||template /* put the ampersands back! */
                end
                else template = NewString||template
            end /* Otherwise */
        end /* select */
    end /* If item <> '' */
end /* While template */

/*say 'FillData End'  Time( "E" ) */

return output


/*:VRX         Get_Env
*/
/* Get the cgi environment variables into g.! */
Get_Env: 

    Global.!Server_software=value('SERVER_SOFTWARE',,'OS2ENVIRONMENT')
    Global.!Server_Name=value('SERVER_NAME',,'OS2ENVIRONMENT')
    Global.!Gateway_Interface=value('GATEWAY_INTERFACE',,'OS2ENVIRONMENT')
    Global.!Server_protocol=value('SERVER_PROTOCOL',,'OS2ENVIRONMENT')
    Global.!Server_Port=value('SERVER_PORT',,'OS2ENVIRONMENT')
    Global.!Request_Method=value('REQUEST_METHOD',,'OS2ENVIRONMENT')
    Global.!Path_Info=value('PATH_INFO',,'OS2ENVIRONMENT')
    Global.!Path_Translated=value('PATH_TRANSLATED',,'OS2ENVIRONMENT')
    Global.!Script_Name=value('SCRIPT_NAME',,'OS2ENVIRONMENT')
    Global.!QUERY_STRING=value('QUERY_STRING',,'OS2ENVIRONMENT')
    Global.!Remote_Host=value('REMOTE_HOST',,'OS2ENVIRONMENT')
    Global.!REMOTE_ADDR=value('REMOTE_ADDR',,'OS2ENVIRONMENT')
    Global.!Auth_Type=value('AUTH_TYPE',,'OS2ENVIRONMENT')
    Global.!REMOTE_USER=value('REMOTE_USER',,'OS2ENVIRONMENT')
    Global.!REMOTE_IDENT=value('REMOTE_IDENT',,'OS2ENVIRONMENT')
    Global.!Content_Type=value('CONTENT_TYPE',,'OS2ENVIRONMENT')
    Global.!Content_Length=value('CONTENT_LENGTH',,'OS2ENVIRONMENT')
    Global.!Auth_Realm=value('AUTH_REALM',,'OS2ENVIRONMENT')



    call RXFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
    call SysLoadFuncs

    t_string = '00'x
    t_object = '01'x
    Global.!Type_string = '00'x
    Global.!Type_Object = '01'x

return

/*:VRX         Get_Template
*/
Get_Template: procedure expose TXN. Global. Data
    /*  Find the template to be used to create the message */
parse arg template

If template == '' then template = TXN.!method

    if template = '' then do
        TXN.!ReturnCode = Global.!code.!notfound
        Global.!message = 'The mail template was not specified (Method)'
        return 0
    end

    /*  Now check to see if the template exists*/
    template= template||'.w3t' 
    strm_status = Stream( template, "Command", "Query Exists" )
    if strm_status = '' then do
        TXN.!ReturnCode = Global.!code.!notfound
        Global.!message = 'The mail template "'TXN.!method'" was not found'
        return 0
    end

    size = Stream( template, "Command", "Query Size" )
    TXN.!Template  = CharIn( template, 1, size )
    strm_status = Stream( template, "Command", "Close" )
return 1

/*:VRX         GetAttrVal
*/
GetAttrVal: 
/* Get an attribute value from a packed object */
/*
    Arguments attribute_name, Object, 'value variable', 'Type variable', 'handle'
    attribute name: String
    Object:         The value of the packed object
    Value variable: String of variable to receive value
    Type variable:  String of variable to receive type of value (string or object)
    handle:         String value 0 or last returned value handle
                    used to unpack lists of similar objects/attributes
    Returns 1 = ok
            0 = notfound/error
*/
parse arg !_attr, !_object, !_retvalue, !_rettype, !_handle
!_attr = Translate(!_attr)
!_len = length( !_object )
!_NOTFOUND = 0
rc=Value(!_retvalue,"")
if !_rettype <> '' then rc=Value(!_rettype,"")
If !_handle = '' then !_position = 0
else !_position =Value(!_handle)
!_start_handle = !_position


/* point past the last value */
If !_position > 0 then do
    !_Itemlen = X2D(C2X( Substr(!_object,!_position,4) ))
    !_position = !_position+!_ItemLen+4
end
else !_position = 1

/* Check for getting header values                   */
/* required to cover case where form contains fields */
/* with the same name as a header value              */
Parse var !_attr !_attr '.' !_H
If !_H == 'H' then !_position = !_len


Do while !_position < !_len
    !_Itemlen = X2D(C2X( Substr(!_object,!_position,4) ))
    If (!_ItemLen+!_position > !_len) | (!_ItemLen < 1) then do
        say 'Bad Object format'
        !_position = !_len /* skip further processing */
    end
    Else do
        !_Item = Substr(!_object,!_position+5,!_ItemLen-1)
        parse var !_Item !_Item '=' !_Value
        If !_Attr == Translate(!_Item) then do
            !_ItemType = Substr( !_object, !_position+4, 1)
            !_NOTFOUND = 1
            rc=value(!_retvalue,!_Value)
            if !_rettype <> '' then rc=Value(!_rettype,!_ItemType)
            if !_handle <> '' then rc=Value(!_handle,!_position)
            !_position = !_len /* skip further processing */
        end
        else do
        end
    end
    !_position = !_position + !_ItemLen +4
end

If (!_NOTFOUND == 0) & (!_Start_handle == 0) then do
    !_Value = '$$'
    select 
    when !_attr = 'PORT' then !_Value = Global.Port
    when !_attr = 'PATH' then !_Value = Global.Path 
    when !_attr = 'DATE' then !_Value = TXN.!Date
    when !_attr = 'TIME' then !_Value = TXN.!Time
    when !_attr = 'SERVER' then !_Value = Global.Server
    when !_attr = 'EM_AUTHOR' then !_Value = Global.EM_Author
    when !_attr = 'EM_Author' then !_Value = Global.EM_Author
    when !_attr = 'EM_TO' then !_Value = Global.EM_Author
    when !_attr = 'SENDER' then !_Value = Global.Sender||'@'Global.Server
    when !_attr = 'REPLY_TO' then !_Value = Global.EM_Reply
    when !_attr = 'MESSAGE_ID' then !_Value = TXN.!message_id
    when !_attr = 'DDMONYYYY' then !_Value = TXN.!DDMonYYYY

    when !_attr = 'OBJECT' then !_Value = TXN.!Object
    when !_attr = 'INSTANCE' then !_Value = TXN.!Instance
    when !_attr = 'METHOD' then !_Value = TXN.!Method
    when !_attr = 'USER' then !_Value = TXN.!User
    when !_attr = 'CLIENTIP' then !_Value = TXN.!ClientIP
    when !_attr = 'REALM' then !_Value = TXN.!Realm
    when !_attr = 'SERVERIP' then !_Value = TXN.!Server_IP
    when !_attr = 'PRESENTATION' then !_Value = TXN.!Presentation
    when !_attr = 'PATH_INFO' then !_Value = TXN.!Path_Info
    when !_attr = 'DOC_PATH' then !_Value = TXN.!Doc_Path
    when !_attr = 'CGI_PATH' then !_Value = TXN.!CGI_Path


    when !_attr = 'FX_AUTHOR' then !_Value = Global.!FaxAuthor
    when !_attr = 'FX_FROMNUMBER' then !_Value = Global.!FaxFromNumber
    when !_attr = 'FX_FROMVOICE' then !_Value = Global.!FaxFromVoice
    otherwise do
        nop
    end
    end
    if !_Value <> '$$' then do
        rc=value(!_retvalue,!_Value)
        if !_rettype <> '' then rc=Value(!_rettype,!_ItemType)
        if !_handle <> '' then rc=Value(!_handle,!_position)
        !_NOTFOUND = 1   
    end
end


objfound:
drop !_attr !_object !_len !_position !_attrlen !_attrType !_Name !_Value !_retvalue !_rettype !_H
return !_NOTFOUND
objerror:
say 'Object not correctly formatted '
return !_NOTFOUND


/*:VRX         GetBody
*/
GetBody: procedure expose Global. debug
parse arg body
/* Read the message body and format as an object */
    /* body holds the data */
    DataSeg = ''        /* initial value of length = 0 */
    /* If there is data in the body, process DataSeg Items */
    type_string = '00'x
    type_object = '01'x

    Do while body <> '' 
        parse var body item '&' body

        If Pos('=', item) <> 0 then do
            /* Translate whitespace to blank */
            item =translate(item, ' ', '+'||'09'x)
            item = HTTP_Escape(item)
            ItemBinaryLen = X2C(D2X(length(item)+1,8))
            DataSeg = DataSeg||ItemBinaryLen||type_string||Item
        end
        else do
            /* we dont have a valid field */
            debug = debug||item 'Invalid (Getbody error)' 
        end
    end /* while body <> '' */

return DataSeg
/*:VRX         Halt
*/
Halt:

exit

/*:VRX         HTTP_Escape
*/
HTTP_Escape: procedure
/*  
    translate HTTP escape characters in the argument string to the corresponding ASCII value 
    sequences consist of %xx where xx is the hex number. We replace the %xx
    with the actual character and move the string up two places
    Returns the changed string
*/
parse arg string
output = ''
do while string <> ''
    parse var string first '%' string
    if string <> '' then do
        hexchar = substr(string,1,2)
        string = substr(string,3)
        first=first||x2c(hexchar)
    end
    output = output||first
end

return output
/*:VRX         Read_Global
*/
Read_Global: 
    /* 
        We need to find the Globals.80 file which could be anywhere! 
        So we have patched the filter to put it in the environment
    */

    Global.!Rules=value('SERVER_RULES',,'OS2ENVIRONMENT')
    file = Global.!Rules||'Globals.'||Global.!Server_Port
    if stream(file, 'c', 'query exists')<>'' then do
        do forever
            strm_status = Stream( file, "Command", "Open Read" )
            if strm_status = 'READY:' then leave
            call SysSleep 1
        end
        GlobLen = Stream( file, "Command", "Query Size" )
        Global.!Globals = Charin(file,1, GlobLen)
        rc=stream(file,'c','close')
    end
    else do
        /* Invalid request, globals file inaccessible */
        return 0
    end
return 1

/*:VRX         Trans_Url
*/
Trans_Url: procedure expose Global.
/* Translate URL into a physical path 
    Arg url to translate
    Returns new url or '' for failed 
    
*/
/*
    Uses a similar structure to that found in the IBM Connection Server 
    Map     source  target      continues   Translates source to target
    User    source  target      Ends scan
    Pass    source  [target]    ends scan   Marks translated url as OK
    Fail    source              ends scan   Marks translated url as disallowed
    Exec    source  target      ends scan   Marks translated url as cgi, target -> cgi directory

where 
    source is the incoming url as modified by prior statements
    target is the text to replace the matching source with

    source may be suffixed by a wildcard '*', If source has a wild card 
    then target may also have one.
    Format of alias entry: command,source,sourcewild,target,targetwild;
    where sourcewild and targetwild are 0 or 1 (1 = wild)
    
    Private user accounts may be translated in the following way:
        Map /~* /userpath/~/web_home/ *
    User is a special case of Pass which will extract the userid following the ~ up to the next /
    and insert it to replace the ~ in the target.
*/
parse UPPER arg url
aliaslist = Global.!Alias
url = '/'||url /* add the leading '/' */
    Do while aliaslist <> ''
        parse UPPER var aliaslist command ',' source ',' swild ',' target ',' twild ';' aliaslist
        command = strip(command)

        Len_source = length(source)
        Len_url = Length(url)
        If swild then do
            test_url = substr(url,1,Len_source)
        end
        Else do
            test_url = url
        end
        
        if test_url == source then do
            /* if the target is wild, append the wild bit */
            If twild then newurl = target||substr(url,Len_source+1)
            Else do
                If target == '' then newurl = source
                Else newurl = target
            end

            /* set up user path is url has '~' */
            If substr(test_url,1,2) == '/~' then do
                /* source == /~ target is required */
                url = substr(url,2)             /* drop the /~ */
                parse var url userid '/' url    /* Get the userid */
                parse var target start '~' end  /* Get the start end end parts of the path */
                newurl = start||userid||'/'||end||url
            end

            select
            when command == 'MAP' then do
                url=newurl
            end
            when command == 'PASS' then do
                url=newurl
                aliaslist = ''  /* finish */
            end
            when command == 'FAIL' then do
                url = ''        /* Say not allowed */
                Global.!message = 'Invalid url for this server'
                aliaslist = ''  /* Finish */
            end
            when command == 'EXEC' then do
                Newurl=target   /* cgi uses this to find the script */
                aliaslist = ''  /* Finish */
                Global.!Iscgi=1 /* say cgi */
            end
            Otherwise do
                aliaslist = ''
                say 'Invalid command' command Newurl
                Newurl = ''
            end
            end 
        end
    end

/* say 'Returning path' Newurl */
return Newurl

