/* Custom mainline for macro */

    call RXFuncAdd "VRLoadFuncs", "VROBJ", "VRLoadFuncs"
    call VRLoadFuncs

    _VREVersion = SubWord( VRVersion( "VRObj" ), 1, 1 )
    if( _VREVersion < 2.12 )then do
        call VRMessage "", "This program requires VX-REXX version 2.1b to run.", "Error!"
        return 32000
    end

    signal on SYNTAX name _VRESyntax
    signal _VREMain

_VRESyntax:
    parse source . . _VRESourceSpec
    call VRMessage "", "Syntax error in" _VRESourceSpec "line" SIGL":" ErrorText(rc), "Error!"
    exit 32000

_VREMain:
/*:VRX         Main
*/
Main:
    signal on halt
/* Process a cgi request for a SIBO object */
/* Object, Method and Instance are found in the Path-Info variable */
/* The incoming data is found in the standard input file */

/* First load the Rexx function library */
    call Load_Rexx

debug = ''
/* DEBUGGING GUIDE 
 to add debugging to this script, enter statements:
debug=debug||value
The value of debug will be appended to the response sent to the client
*/
/* set up some global values */
    Global.!Server=Value('SERVER_NAME',,"OS2ENVIRONMENT")
    Global.!ServerIP=Value('SERVER_ADDR',,"OS2ENVIRONMENT")
    If Global.!ServerIP == '' then do
        /* look up the ip number from the name */
        if SockGetHostByName(Global.!Server,"host.!") then do
            Global.!ServerIP = host.!addr
        end
    end

/* check to see if IF_DomainList environment variable exists */
    Global.!DomainList=value('IF_DomainList',,'OS2ENVIRONMENT')
    If Global.!DomainList == '' then do
        /* Load the global values from cgi-sibo.cf */
        rc = GetFileGlobals() /* load the object list from .cf file */
    end
    Else do
        rc = GetEnvGlobals() /* Load the object list from the environment */
    end
/* results in Global.!Objects being loaded */


/* Get some standard environment variables */
    uri = Value('PATH_INFO',,"OS2ENVIRONMENT")
    Global.!userid=Value('REMOTE_USER',,"OS2ENVIRONMENT")
    Global.!who=Value('REMOTE_ADDR',,"OS2ENVIRONMENT")
    Global.!realm=Value('AUTH_REALM',,"OS2ENVIRONMENT")
    Global.!verb=Value('REQUEST_METHOD',,"OS2ENVIRONMENT")
    Global.!port =Value('SERVER_PORT',,"OS2ENVIRONMENT")
    Global.!Path_Translated=VALUE('PATH_TRANSLATED',,"OS2ENVIRONMENT")
    Global.!Path_Info=uri
    /* Just take up to the last '/' */
    position  = LastPos( "/", Global.!Path_Info )
    Global.!Doc_path = substr(Global.!Path_Info,1,position)

    Global.!Script_Name=Value('SCRIPT_NAME',,"OS2ENVIRONMENT")
    /* Just take up to the last '/' */
    position  = LastPos( "/", Global.!Script_Name )
    Global.!cgi_path = substr(Global.!Script_Name,1,position)


/* set up some other globals */
    Global.!endline = '0d0a'x
    Global.!type_string = '00'x
    Global.!type_object = '01'x
    Global.!Indent='    '

/*  Get Object, Instance, Method from Path-Info */
    /* First remove any suffix */
    position  = LastPos( "$", uri )
    If position = 0 then do
        Global.!Error = 'NotFound'
        Global.!Message = 'Incorrect URL format'
        Signal ProcessError /* Resource Not found */
    end
    position  = Pos( ".", uri, position )
    if position <> 0 then do
        suffix  = SubStr( uri, position + 1 )
        uri = SubStr(uri,1,position -1)
    end

    /* Remove the slashes */
    position  = Pos( "/", uri, 1 )
    do while position <> 0
        uri = DelStr( uri, position, 1 )
        position  = Pos( "/", uri, 1 )
    end

    /* Now get the Object, Instance and Method values */
    parse var uri dir '$' Object '$' Instance '$' Method '$' .
    /* Remove any slashes */
    
/*  Find Object - Requires Enviroment variable IF_Objects_cgi */
    Sockdef = FindObj(Object)
    if Sockdef = '' then do
        Global.!Error = 'NotFound'
        Global.!Message = 'Object 'object' not found'
        Signal ProcessError /* Resource Not found */
    end

    type_string = '00'x
    type_object = '01'x


/* Build Standard SIBOM Header */
    HdrSeg = ''
    rc = AddAttrVal('Object','HdrSeg',Object,type_string)
    rc = AddAttrVal('Instance','HdrSeg',Instance,type_string)
    rc = AddAttrVal('Method','HdrSeg',Method,type_string)
    rc = AddAttrVal('User','HdrSeg',Global.!userid,type_string)
    rc = AddAttrVal('ClientIP','HdrSeg',Global.!who,type_string)
    rc = AddAttrVal('Realm','HdrSeg',Global.!realm,type_string)
    rc = AddAttrVal('ServerIP','HdrSeg',Global.!ServerIP,type_string)
    rc = AddAttrVal('Presentation','HdrSeg','HTML',type_string)
    rc = AddAttrVal('Path_Info','HdrSeg',Global.!Path_Info,type_string)
    rc = AddAttrVal('Doc_Path','HdrSeg',Global.!Doc_Path,type_string)
    rc = AddAttrVal('cgi_Path','HdrSeg',Global.!cgi_Path,type_string)

/* Read input data and build SIBOM data */
    If Global.!verb = 'POST' then do
        len =Value('CONTENT_LENGTH',,"OS2ENVIRONMENT")
        dataseg = ''

        do while length(dataseg) < len         /* ++++ */
            dataseg = dataseg||linein()
        end        
        dataseg = GetBody(dataseg)
    end
    Else do
        parse arg dataseg /* Get the argument */
        dataseg = strip(dataseg)
        If dataseg == '' | dataseg == '""' then do
            /* get the Query_string value from the environment */
            dataseg=Value('QUERY_STRING',,"OS2ENVIRONMENT")
        end
        
        dataseg = GetBody(dataseg)
    end
/* debug=debug||'<P><PRE>'||Unpack2Text(dataseg)||'</PRE>' */
/* Send message to Object */
    rc = SockMsg(SockDef, HdrSeg, DataSeg)
    if rc <> 1 then signal ProcessError /* +++ ??? */

/* Create the Response URI */
    /* Put the result in the page */
    /* Process the header fields */
    /* The reply header contains:
        Object 
        Instance    Unique ID of reply
        Method      template name 
        Type        SIBO/template or html type
        Data        The packed data object
    */
    /* Create the URI */
    parse var Global.!RelPath first '$' oldobject '$' oldinstance '$' oldmethod '$' rest
    Object = GetHdr('OBJECT')
    Instance = GetHdr('INSTANCE')
    Method = GetHdr('METHOD')
    ReturnCode = GetHdr('RETURNCODE')
    If ReturnCode = '*RETURNCODE*' then ReturnCode = 200
    NewObject = '$'Object'$'Instance'$'Method'$'
    Parturi = first||NewObject||rest    
    Newuri = AddHTTP(Parturi)
/*    call lineout ,'HTTP/1.0 'ReturnCode  -- Apache does not like this format */
    call lineout ,'STATUS:'ReturnCode
    call lineout ,'URI: <'Newuri'>'

/* +++ Causes 404 not found in IBM Connection Server (causes a Redirect?)*/
/*    call lineout ,'Location: 'Newuri    */

    time = Time( "N" )
    date = Date( "S" )
    ttl = GetHdr('TIMETOLIVE')  /* Time to live in minutes */
    if ttl = '*TIMETOLIVE*' then ttl = 30
    http_date = AddMinutes(ttl, date, time)
    call lineout ,'Expires: 'http_date

/* Now check the type and method and act accordingly */
    type = GetHdr('TYPE')
    response = GetHdr('DATA')

    SELECT
/* If TYPE=x-sibom/debug, process the object into plain text */
    when (type = 'application/x-sibom') | (type = 'xSIBOM/template') then do
        /* Process the input and the response into html */
        Global.!Indent = ''
        If _ItemType = Global.!Type_Object then response = Unpack2Text(response)
        response = 'Object:'object '<P>'  ,
            'Instance:' instance '<P> Method:' method '<P>'  ,
            '<pre>'||response||'</pre>'

        response = '<HTML><HEAD><TITLE>SIBOM debug 'Newuri'</TITLE></HEAD>' ,
            '<BODY><H1>SIBO debug</H1><H2>'Newuri'</H2>' ,
            '<H2> Transaction 'txnumber' </H2><P>'  ,
            response||'</BODY></HTML>'

        type='text/html'
    end

/* Otherwise just return the data */
/* return the data to the client */
    otherwise do
        /* type=type */
    end

    END /* SELECT */


/* return error message */
call lineout ,'CONTENT-TYPE: text/html'
call lineout ,''
call lineout ,response
If debug <> '' then call lineout ,debug
call lineout

return 0

ProcessError:
response = Global.!Message
/* return error message */
call lineout ,'STATUS:502 Bad Gateway'
call lineout ,'CONTENT-TYPE: text/plain'
call lineout ,''
call lineout ,response
If debug <> '' then call lineout ,debug
call lineout

return 0
/*:VRX         AAAVersion
*/
AAAVersion: 
Global.!version = '1.06'
return

/*:VRX         AddAttrVal
*/
AddAttrVal: 
/* Add an object or item to a packed object */
parse arg !_attr, !_object, !_Value, !_Type
/* 
    Arguments
        attr -  Literal name of the attribute
        object- Quoted name of the variable containing the object
        value-  value the attribute is to be set to
        type -  Data type of the attribute (string or object)
    Returns
        1 OK
        0 Error
*/


    /* Add the new attribute */
    !_Item = !_attr||'='||!_value
    !_ItemBinaryLen = X2C(D2X(length(!_Item)+1,8))
    !_NewAttr = !_ItemBinaryLen||!_type||!_item
    rc=Value(!_object,Value(!_object)||!_NewAttr)
    !_FOUND = 1


drop !_attr !_object !_Value !_Type
drop !_ItemBinaryLen !_NewAttr !_Item 
return !_FOUND

/*:VRX         AddHTTP
*/
AddHTTP: procedure expose Global.
parse arg uri
/*
look for protocol and server. if either is missing,
add the appropriate values
*/
    parse var uri protocol ':' uri
    if uri = '' then do
        uri = protocol
        protocol = 'http'
    end
    parse var uri _uri '//' server '/' uri
    if uri = '' then do
        uri = _uri 
        server = Global.!server
        if Global.!port <> 80 then server=server||':'Global.!port
    end
if left(uri,1) = '/' then uri = substr(uri,2)
return protocol||'://'||server||'/'||uri

/*:VRX         AddMinutes
*/
AddMinutes: procedure 
/* adds a number of minutes to the time and */
/* returns date in format Day, dd-Mon-yyyy hh:mm:ss */
parse arg ttl, date, time
parse var time hh ':' mm ':' ss
Months = 'Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec'
DaysOfWeek = 'Mon Tue Wed Thu Fri Sat Sun'
mon.1=31
mon.2=28
mon.3=31
mon.4=30
mon.5=31
mon.6=30
mon.7=31
mon.8=31
mon.9=30
mon.10=31
mon.11=30
mon.12=31

mm = mm+ttl
hhs = mm%60
mm = mm//60
hh = hh+hhs
dys = hh%24
hh = hh//24

yyyy = substr(date,1,4)
mn = substr(date,5,2)
dd = substr(date,7,2)
mn = mn + 0 /* Force to single digit integer */
dds = dd + dys
days = Date( "B" ) + dys
weekday = days//7
DayOfWeek = word(DaysOfWeek,weekday+1)
do while dds > 0
    div = mon.mn
    if mn = 2 then div=div+leap(yyyy)
    if dds > div then do
        mn = mn + 1
        dds = dds-div
        if mn > 12 then do
            mn = 1
            yyyy=yyyy+1
        end
    end
    else do
        dd = dds
        dds = 0
    end
end
string = DayOfWeek', 'dd Word(Months, mn) yyyy hh':'mm':'ss 'GMT'
return string

/*:VRX         FindObj
*/
FindObj: procedure expose Global.
parse arg object

_Object = Translate(Object)
socket = ''
search = Global.!Objects
do while search <> ''
    parse var search objdef ';' search
    parse var objdef domain objname host port crypt key
    if _object = Translate(objname) then do
       socket = host||' 'port 
       search = ''
    end
end

return socket

/*:VRX         GetAttrVal
*/
GetAttrVal: 
/* Get an attribute value from a packed object */
/*
    Arguments attribute_name, Object, 'value variable', 'Type variable', 'handle'
    attribute name: String
    Object:         The value of the packed object
    Value variable: String of variable to receive value
    Type variable:  String of variable to receive type of value (string or object)
    handle:         String value 0 or last returned value handle
                    used to unpack lists of similar objects/attributes
    Returns 1 = ok
            0 = notfound/error
*/
parse arg !_attr, !_object, !_retvalue, !_rettype, !_handle
!_attr = Translate(!_attr)
!_len = length( !_object )
!_NOTFOUND = 0
rc=Value(!_retvalue,"")
if !_rettype <> '' then rc=Value(!_rettype,"")
If !_handle = '' then !_position = 0
else !_position =Value(!_handle)
!_start_handle = !_position

/* point past the last value */
If !_position > 0 then do
    !_Itemlen = X2D(C2X( Substr(!_object,!_position,4) ))
    !_position = !_position+!_ItemLen+4
end
else !_position = 1

Do while !_position < !_len
    !_Itemlen = X2D(C2X( Substr(!_object,!_position,4) ))
    If (!_ItemLen+!_position > !_len) | (!_ItemLen < 1) then do
        say 'Bad Object format'
        !_position = !_len /* skip further processing */
    end
    Else do
        !_Item = Substr(!_object,!_position+5,!_ItemLen-1)
        parse var !_Item !_Item '=' !_Value
        If !_Attr == Translate(!_Item) then do
            !_ItemType = Substr( !_object, !_position+4, 1)
            !_NOTFOUND = 1
            rc=value(!_retvalue,!_Value)
            if !_rettype <> '' then rc=Value(!_rettype,!_ItemType)
            if !_handle <> '' then rc=Value(!_handle,!_position)
            !_position = !_len /* skip further processing */
        end
        else do
        end
    end
    !_position = !_position + !_ItemLen +4
end


objfound:
drop !_attr !_object !_len !_position !_!_attrlen !_!_attrType !_Name !_Value !_retvalue !_rettype
return !_NOTFOUND
objerror:
say 'Object not correctly formatted '
return !_NOTFOUND


return

/*:VRX         GetBody
*/
GetBody: procedure expose Global. debug
parse arg body
/* Read the message body and format as an object */
    /* body holds the data */
    DataSeg = ''        /* initial value of length = 0 */
    /* If there is data in the body, process DataSeg Items */
    type_string = '00'x
    type_object = '01'x

    Do while body <> '' 
        parse var body item '&' body

        If Pos('=', item) <> 0 then do
            /* Translate whitespace to blank */
            item =translate(item, ' ', '+'||'09'x)
            item = HTTP_Escape(item)
            ItemBinaryLen = X2C(D2X(length(item)+1,8))
            DataSeg = DataSeg||ItemBinaryLen||type_string||Item
        end
        else do
            /* we dont have a valid field */
            debug = debug||item 'Invalid (Getbody error)' 
        end
    end /* while body <> '' */

return DataSeg

/*:VRX         GetEnvGlobals
*/
GetEnvGlobals: procedure expose global.
        domain = ''
        Do i = 1 to words(Global.!DomainList)
            wd = Word(Global.!DomainList,i)
            parse var wd ip ',' domain ',' domainName
            if ip == Global.!ServerIP then leave
        end
        Global.!Objects=value('IF_Objects_'domain,,'OS2ENVIRONMENT')

return 1

/*:VRX         GetFileGlobals
*/
GetFileGlobals: procedure expose Global.

    Global.!dir = directory() /* get the current directory */
    dir=Global.!dir||'\cgi-sibo.cf'

    Global.!DomainList = ''
    Global.!Objects = ''

    if stream(dir, 'c', 'query exists')<>'' then do
        strm_status = Stream( dir, "Command", "Open Read" )
        if strm_status <> 'READY:' then leave
        GlobLen = Stream( dir, "Command", "Query Size" )
        Glob = Charin(dir,1, GlobLen)
        rc=stream(dir,'c','close')
        /* Remove any CR/LF and tab codes in the variables */
        Glob = Translate(Glob,,'090a0d'x)
        parse var Glob 'Domains:' Global.!DomainList ':EndDomains'
    end
    /* Glob has the file contents, Global.domainList has the domains */

    Do i = 1 to words(Global.!DomainList)
        wd = Word(Global.!DomainList,i)
        parse var wd ip ',' domain ',' domainName
        if ip == Global.!ServerIP then leave
    end

    /* domain contains the domain correponding to the request */
    /* extract the related set of objects */
    interpret "parse var Glob 'Objects_"domain':'"' Global.!Objects ':EndObjects_"domain"'"
    Global.!Objects = Translate(Global.!Objects,,'090a0d'x)

    drop Glob Globlen strm_status rc wd ip domain
return 1

/*:VRX         GetHdr
*/
GetHdr: 
/* return the value of a header item */
parse arg aitem
handle = 0
If GetAttrVal(aitem, Global.!reply,'GetHdr!_value','_ItemType','handle') then return GetHdr!_value
else return '*'||aitem||'*'


/*:VRX         Halt
*/
Halt:

exit

/*:VRX         HTTP_Escape
*/
HTTP_Escape: procedure
/*  
    translate HTTP escape characters in the argument string to the corresponding ASCII value 
    sequences consist of %xx where xx is the hex number. We replace the %xx
    with the actual character and move the string up two places
    Returns the changed string
*/
parse arg string
output = ''
do while string <> ''
    parse var string first '%' string
    if string <> '' then do
        hexchar = substr(string,1,2)
        string = substr(string,3)
        first=first||x2c(hexchar)
    end
    output = output||first
end

return output

/*:VRX         Leap
*/
 
Leap: procedure
parse arg yyyy
noleap = 1
leap = 0
leap0 = yyyy//400 
if leap0 <> 0 then do 
    leap0 = yyyy//4
    if leap0 = 0 then noleap = yyyy//100
end
if noleap = 0 then leap = 0
else if leap0 = 0 then leap = 1

return leap



/*:VRX         Load_Rexx
*/
Load_Rexx: 
 
if RxFuncQuery("SockLoadFuncs") then do
    rc = RxFuncAdd("SockLoadFuncs","RxSock","SockLoadFuncs")
    rc = SockLoadFuncs()
end
if RxFuncQuery("SysLoadFuncs") then do
    rc = RxFuncAdd("SysLoadFuncs","RexxUtil","SysLoadFuncs")
    rc = SysLoadFuncs()
end

return

/*:VRX         SockMsg
*/
SockMsg: Procedure expose Global.
    parse arg SockDef, HdrSeg, DataSeg


/* Open a socket to the Server */

    Parse var SockDef server port

/*---------------------------------------------------------------
 * open socket
 *---------------------------------------------------------------*/
socket  = SockSocket("AF_INET","SOCK_STREAM",0)
if (socket = -1) then
   do
   Global.!message = "Error on SockSocket:" errno
   signal donerr
   end


/*---------------------------------------------------------------
 * connect socket
 *---------------------------------------------------------------*/
server.!family = "AF_INET"
server.!port   = port
server.!addr   = server

rc = SockConnect(socket,"server.!")
if (rc = -1) then
   do
   Global.!message = "Error on SockConnect:" errno
   signal donerr
   end


/* Get some data to send over the socket */
HdrLen = X2C(D2X(length(HdrSeg),8))

rc = SockSend(socket,HdrLen)
if (rc = -1) then
   do
   Global.!message = "Error on SockSend:" errno
   signal donerr
   end

rc = SockSend(socket,HdrSeg)
if (rc = -1) then
   do
   Global.!message = "Error on SockSend:" errno
   signal donerr
   end

DataLen = X2C(D2X(length(DataSeg),8)) /* Build the length part of the message */
rc = SockSend(socket,DataLen)
if (rc = -1) then do
   Global.!message = "Error on SockSend:" errno
   signal donerr
end

DataLen = Length(DataSeg)
If DataLen > 0 then do
    Do while DataLen > 1500
        DataPart = substr(DataSeg,1,1500)
        DataSeg = Substr(Dataseg,1501)
        DataLen = DataLen - 1500
        rc = SockSend(socket,DataPart)
        if (rc = -1) then do
            Global.!message = "Error sending data to object, error code:" errno
            signal donerr
        end
    end
    rc = SockSend(socket,DataSeg)
    if (rc = -1) then do
        Global.!message = "Error sending data to object, error code:" errno
        signal donerr
    end
end

/*------------------------------------------------------------------
 * receive the result from the server
 *------------------------------------------------------------------*/

/* get the data segment length */
    rc = SockRecv(socket,"reply",4 )
    if (rc = -1) then
       do
       Global.!message = "Error on SockRecv:" errno
       signal donerr
      end
    if rc < 4 then do 
        Global.!message = "Error response too short"
        signal donerr
    end
Global.!reply = ''
/* Convert bin_len to char */
len = X2D(C2X( reply ))

do while len > 0
    rc = SockRecv(socket,"reply",1500 )
    if (rc = -1) then
       do
       Global.!message = "Error receiving data from object, error code:" errno
       signal donerr
      end
    Global.!reply = Global.!reply||reply
    len = len - rc
end

rc = SockSoClose(socket)
if (rc = -1) then
   do
   Global.!message = "Error on SockSoClose:" errno
   return 0
   end
return 1
donerr:
rc = SockSoClose(socket)
return 0
/*:VRX         Unpack2Text
*/
Unpack2Text: procedure expose Global.
parse arg object
    Text = ''
    ObjLen = Length(object)
    /* Get the item values */
    position = 1    /* Point past Header Length */
    itemno = 1
    indent = Global.!indent
    Global.!indent = Global.!Indent||'    '
    do while position <= ObjLen
        ItemLen = X2D(C2X( Substr(object,position,4) ))
        If (ItemLen > ObjLen-position) | (ItemLen <= 0) then do
            Text = text||Global.!endline||'Error Invalid object!!!'||global.!newline||c2x(substr(object,position))
            Position = ObjLen+1
        end
        Else do
            ItemType = substr(object,position+4, 1)
            Item = substr(object, position+5, ItemLen-1)
            parse var Item Name '=' Value 
            If ItemType = Global.!type_object then do
                Global.!indent = Global.!Indent||'    '
                Text = Text||indent Name||'='||Global.!endline||Unpack2Text(Value)
                Global.!indent = indent
            end
            Else do
                Text = Text||indent||Name||'='||value||Global.!endline
            end
            position = position + ItemLen +4
        end
    end
    Global.!indent = indent
return Text

