/* Configuring multi host using slip */
/* 
first configure slip.cfg in mptn/etc as follows:

interface sl0 { device="com2" }
interface sl1 { device="com2" }
interface sl2 { device="com2" }
interface sl3 { device="com2" }
interface sl4 { device="com2" }
interface sl5 { device="com2" }
interface sl6 { device="com2" }
interface sl7 { device="com2" }
interface sl8 { device="com2" }
interface sl9 { device="com2" }
*/

/* Modify the following as required */
lan_card = 0	/* The Lan interface number to publish the arp addresses */
gateway='65'	/* last part of router IP number on your network */
slipstart = '212'	/* ONE LESS than first IP number to be assigned - must have space for 20 */
slipcount = 10	/* Number of slip ports to assign */
slipcom = 'com2'	/* comment only, set in slip.cfg, set to '' to skip slip */
pppcom = 'com3' /* Used for the ppp command */
pppcount = 10	/* Number of PPP ports to assign */
/* Do not change this entry */
pppstart = slipstart+slipcount

/* Load the REXX function Library */
Call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
Call SysLoadFuncs
Call RxFuncAdd 'SockLoadFuncs' ,'RxSock', 'SockLoadFuncs'
Call SockLoadFuncs

say 'Wait 5 seconds for TCP/IP to start'
call SysSleep 5

'@echo=off' 

/* Set the variables up */
ipnumber = Get_IP()
say 'IPNUMBER:'ipnumber
position = LastPos('.',ipnumber)
Network = Substr(ipnumber,1,position)
IPnumber = Substr(ipnumber,position+1)

say 'Network IP is 'Network', this host is 'Network||IPnumber

/* Get the adapter internal address for the lan_card interface */
id = Get_Enet( lan_card )
lan_id = substr(id,1,2)':'substr(id,3,2)':'substr(id,5,2)':'substr(id,7,2)':'substr(id,9,2)':'substr(id,11,2)

say 'ARP entries will be published  on Lan Card:'lan_id

say 'Wait 10 seconds for testing, Ctrl+Break to cancel'
call SysSleep 10

/* Do the setup.cmd bit */
	say 'route -fh'
	say 'arp -f'
	say
	say 'ifconfig lan0 'Network||ipnumber' netmask 255.255.255.0'
	say 'route add default 'Network||gateway' 1'
	say 'ipgate on'
	say

/* Configure the loopback interface (if not already done */
say 'Configuring local loopback'
	    'ifconfig lo 127.0.0.1'
say


If slipcom = '' then signal do_ppp



/* then start slip */
say 'Starting SLIP'
'start slip' 
say 'slip:'rc
/*'detach slip'*/
/* wait a few seconds for SLIP to start */
say 'waiting 10 seconds'
call syssleep 10


/* Now configure the slip interfaces */
Do i = 1 to slipcount
say 'Configuring port 'Network||slipstart+i' on Slip 'i-1
	'ifconfig sl'i-1' 'Network||slipstart+i'  127.0.0.1'
	'route delete 127.0.0.1 'Network||slipstart+i /* delete spurious route */
end
say
Do i = 1 to slipcount
say 'Configuring ARP 'Network||slipstart+i' on Lan 'lan_id
	/* Now publish the ip address on ethernet */
	'arp -s 'Network||slipstart+i lan_id' pub'

	/* and now add the appropriate routes */
	'route add host 'Network||slipstart+i' 127.0.0.1 0'
end
say
say
/* slip can now be cancelled */
say 'Please kill slip'
pause

Do_ppp:

/* The following PPP code kindly supplied by: */
/* Author : Al Nichols                                Language: REXX    */
/* modified: Derek Sims, October 1996*/
/* Title  : Virtual2.cmd                              Date    : 07Jun95 */
/* Purpose: Configure all available interfaces for virtual IP addresses */
/*          outside of ppp0 which is the main(real) link                */
/* IO     : the slip command uses the file slip.cfg in the tcpip\bin dir*/
/* Changes: use point to point instead of slip to make it self contained*/

/* Now start the PPP interfaces */
do index = 1 to pppcount
/*   "start ppp silent "pppcom" 28800 mru 1500 priority 1"*/
   say 'Starting ppp 'pppcom 'for' Network||pppstart+index
   "start /N /MIN ppp silent "pppcom" 28800 mru 1500 priority 1"
   rc=syssleep(4)
end

/* Now configure the ppp interfaces */
Do i = 1 to pppcount
say 'Configuring port 'Network||pppstart+i' on ppp 'i-1
	'ifconfig ppp'i-1' 'Network||pppstart+i'  127.0.0.1'
	'route delete 127.0.0.1 'Network||pppstart+i  /* delete spurious route */
end
say

Do i = 1 to pppcount
say 'Configuring ARP 'Network||pppstart+i' on Lan 'lan_id
	/* Now publish the ip address on ethernet */
	'arp -s 'Network||pppstart+i lan_id' pub'

	/* and now add the appropriate routes */
	'route add host 'Network||pppstart+i' 127.0.0.1 0'
end
say


'netstat -r'
'arp -a'

/* Testing */
say 'Ready to start servers, press Ctrl/c to cancel'
'Pause'


/* The following code may be deleted. I use it to start */
/* various WWW, FTP anf other servers that must be */
/* started after the IP addresses are set                       */


/* start the web server */
say 'starting web server'
start Golive.cmd
call syssleep 5

/* start the ftp server */
say 'starting FTP server'
start 'D:\penguin_ftp\penguin.cmd'
call syssleep 5

/* start the count server */
say 'Starting WWW Count server'
'd:'
'cd \count_server'
start 'D:\count_server\ctserver.exe d:\count_server\gparam'
call syssleep 5

/* Start Mail server */
say 'Starting Mail SMTP and POP server'
start 'D:\InetMail\InetMail.exe'

say 'Exiting'
return

/*************************************************/
/* Do not delete the following procedures */
/*                                                               */

Get_IP: procedure
/* Return the this host's IP address */

return SockGetHostID()


Get_Enet:
/*
    NOTE This does NOT work if multiple interface are defines, netstat hangs for input
	Returns the first Ethernet adapter internal address 
*/
parse arg number
   PUSH '                    '
    'netstat -n > temp2'
    notfound = 1
    data = ''
    Do while lines('temp2') 
        data = data||linein('temp2')
    end
    call lineout 'temp2' /* Close temp */

    Do while data <> '' & notfound
        data = Strip(data)
        parse var data . 'Interface' string 'MTU' data
/* NOTE various versions of netstat give slightly different formats*/
/* Please check and change as required */
        Parse var string LanCard':' . 'address' ENet .
        If LanCard = number then do
            return ENet
            notfound = 0
        end
    end
say 'Error Lan Interface 'number' Not Found'
return ''
/* End of program */
/* pipe char = | */