/* Custom mainline for macro */

    call RXFuncAdd "VRLoadFuncs", "VROBJ", "VRLoadFuncs"
    call VRLoadFuncs

    _VREVersion = SubWord( VRVersion( "VRObj" ), 1, 1 )
    if( _VREVersion < 2.12 )then do
        call VRMessage "", "This program requires VX-REXX version 2.1b to run.", "Error!"
        return 32000
    end

    signal on SYNTAX name _VRESyntax
    signal _VREMain

_VRESyntax:
    parse source . . _VRESourceSpec
    call VRMessage "", "Syntax error in" _VRESourceSpec "line" SIGL":" ErrorText(rc), "Error!"
    exit 32000

_VREMain:
/*:VRX         Main
*/
Main:
    signal on halt
    call Load_DLLs

/* 
Script to process the IntrFilt version of the Goserve log file
and produce NCSA format logs
*/

/*
    args:   port number - required to find the correct goarch file

    environment: uses the IntrFilt environment variables to determine how to process the data
*/

parse arg port
port = strip(port)
call lineout ,'Port is:"'port'"'

/* Set up some basic values for file names */
date = Date('S')
Time = Time('M') /* in minutes */
Time = Translate(time,'_',':')
archive = date'.'time
Day = Date( "W" )

/* Only run once per week */
If Day <> 'Sunday' then exit



/* get a list of all the serv IP numbers into servers */

/* format serverIP,root_directory; */
/*
    The IP number is used to separate the server log by domain,
    and the home directory is the root to store the log file, and to run wusage
*/
ServerList = GetServers()

call lineout ,'Log copied to 'archive'.'port
'Rename Goarch.'||port' 'archive||'.'||port
/* Build a list of server IPs */
servers = ''
temp = ServerList
HomeIP = SockGetHostID()
call lineout ,'Home IP:'HomeIP
Do while temp <> ''
    parse var temp ip ',' path ';' temp
    If ip = HomeIP then IP = '' /* A feature of goserve - home IP is not recorded !! */
    servers = servers||ip||';'
end
call lineout ,'Server:'servers
'Bld_Log  'archive'.'port servers 


temp = ServerList
do while temp <> ''
    parse var temp IP ',' path ';' temp
    ip = strip(ip)
    path = strip(path)
    If ip = HomeIP then ip = 'server'
    ip=translate(ip,'_','.')
    path = Translate(Path,'\','/')
    say path
    logpath = ''
    /* Check the log path exists, create it if necessary */
    rc = SysFileTree(path'stats\logs','dirList','D')
    If rc == 0 then Do index = 1 to dirList.0
    say dirList.index
        parse var dirList.index date time size attr name
        name = strip(name)
        say name
        If Translate(name) == Translate(path'stats\logs') then do
            logpath = name
            index = dirList.0
        end
    end
    If logpath == '' then do 
        /* Directory not found, create */
        rc = SysMkDir(path'stats')
        rc = SysMkDir(path'stats\logs')
        If rc <> 0 then do
            say 'unable to create directory 'path'stats\logs'
            say ' log file ' ip||'.log not copied'
        end
        else logpath = path'stats\logs'
    end

    /* Now copy the log data */
    If Logpath <> '' then do
        call lineout ,'copy 'ip||'.log 'path'stats\logs\'archive'.log'
        'copy 'ip||'.log 'path'stats\logs\'archive'.log'
        say 'copy rc:'rc
        If rc == 0 then  do
            'del 'ip||'.log'
            /* Run wusage if found in stats directory */
            usagePath = path'stats\wusage.cf'
            strm_status = Stream( usagePath, "Command", "Query Exists" )
            If strm_status <> '' then do
                'wusage -c 'usagePath
            end
            else say 'wusage.cf does not exists for 'IP' on 'usagePath
        end
        else say 'Copy failed, 'ip'.log not copied' 

    end /* If LogPath */
end /* While temp */

exit

/*:VRX         GetServers
*/
GetServers: 
/* Scan the IntrFilt Environment and build a list of server IPs, and their root directories */

    servers = ''

    Global.!DomainList=value('IF_DomainList',,'OS2ENVIRONMENT')
    Global.!DomainList = Translate(Global.!DomainList,' ','0a0d09'x)

call lineout ,'Domains:'Global.!DomainList

    Do i = 1 to words(Global.!DomainList)
        wd = Word(Global.!DomainList,i)
        parse var wd ip ',' short ',' domainName
call lineout ,'"'ip'" "'short'" "'domainname'"'
        Global.!Alias=value('IF_Alias_'short,,'OS2ENVIRONMENT')
        Global.!Alias = Translate(Global.!Alias,' ','0a0d09'x)
        /* Find the last entry, should look like 'Pass,/,' */
        Parse var Global.!Alias . 'Pass,/,' star1 ',' path ',' star2 ';' .
        servers = servers||ip||','||path||';'

        call lineout ,'Server:'ip||','||path

    end

return servers

/*:VRX         Halt
*/
Halt:

exit

/*:VRX         Load_DLLs
*/
Load_DLLs: 
   if RxFuncQuery("SockLoadFuncs") then
      do
      rc = RxFuncAdd("SockLoadFuncs","RxSock","SockLoadFuncs")
      rc = SockLoadFuncs()
      end
   rc = RxFuncAdd("SysLoadFuncs","RexxUtil","SysLoadFuncs")
   rc = SysLoadFuncs()


return

